from typing import Dict, List
import logging


import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

default_logger = logging.getLogger(__name__)


class Zabbix:
    def __init__(self, url: str, username: str, password: str, logger=default_logger):
        self.url = url
        self.username = username
        self.password = password
        self.token = None

        self.logger = logger

    def call_api(self, method: str, params: Dict[str, str]):
        if method != "user.login" and self.token is None:
            self.login()

        params = {
            "jsonrpc": "2.0",
            "method": method,
            "params": params,
            "id": 1,
            "auth": self.token,
        }
        try:
            self.logger.debug(f"Calling the API with params: {params}")
            self.logger.info(f"Using requests from {requests.__file__}")
            r = requests.post(self.url, json=params, verify=False)
            self.logger.debug(f"Got API result {r}")
            if r.status_code >= 300:
                self.logger.error(f"Error calling API: {r}: {r.text}, params: {params}")
            return r.json()["result"]
        except Exception as e:
            self.logger.error(f"Error calling API: {self.url}: {params}: {e}")
            return []

    def login(self):
        params = {"user": self.username, "password": self.password}
        self.token = self.call_api("user.login", params)

    def get_hosts(self, groups: List[str]):
        params = {
            "user": self.username,
            "password": self.password,
            "groupids": groups,
            "selectGroups": "extend",
            "selectInterfaces": "extend",
            "output": ["name", "status", "host", "hostid"],
        }
        return self.call_api("host.get", params)

    def get_events(self, event_ids=None):
        params = {
            "user": self.username,
            "password": self.password,
            "selectHosts": "extend",
            "eventids": event_ids,
            "filter": {"object": "0", "value": "1"},
        }

        if event_ids is not None:
            params["eventids"] = event_ids

        return self.call_api("event.get", params)

    def get_problems(self):
        params = {
            "user": self.username,
            "password": self.password,
            "filter": {"object": "0"},
        }
        return self.call_api("problem.get", params)

    def get_triggers(self):
        params = {
            "user": self.username,
            "password": self.password,
            "filter": {"value": "1"},
            "selectHosts": "extended",
        }
        return self.call_api("trigger.get", params)


def main():
    import json

    default_logger.setLevel(logging.DEBUG)
    st = logging.StreamHandler()
    st.setLevel(logging.DEBUG)
    default_logger.addHandler(st)

    with open("properties.json", "r") as f:
        config = json.load(f)

    z = Zabbix(config["zabbix_url"], config["zabbix_username"], config["zabbix_password"])

    problems = z.get_problems()
    events = z.get_events(event_ids=[problem["eventid"] for problem in problems])
    for event in events:
        print(event)


if __name__ == "__main__":
    main()
